import simu.*;
import java.util.Vector;
import java.awt.Graphics;
import java.awt.image.*;
import java.awt.Image;

import java.awt.Color;



/**
 * Class extending SimuApplet. It is used to display the delaunay triangulation on the sphere.
 */
public class Display extends SimuApplet {
	
    private int nPoint;

    
    private int zoom;

    private String projec;
    private String gType;

    private int mode;

    private int nProj;
    private Grid grid;
    private Vector cellList;

	
    /**
     * Constructor of Display inheriting SimuApplet
     */
    public Display() {
	appletDoc("delaunay_doc.xml");
    }

    /**
     * Implementation of the method "calcul"
     * @param in ListeValeurs de Simulab
     * @return out ListeValeurs pour Simulab 
     */
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {

	double facteur = 100;

	int offsetX = (int)(3.0*facteur);
	int offsetY = (int)(1.8*facteur);
        
	double th0 = 0.0;
	double ph0 = 0.0;

	Image img = createImage(600, 400);
	Graphics g = (img).getGraphics();

	ListeValeurs out = new ListeValeurs();

	nProj=0;
	nPoint=in.lireEntier("nPoint");

	
	
	zoom = in.lireEntier("zoom");
	double dilat = facteur*zoom;
		
	
	String gType = in.lireString("gType");
	    

	if(gType.equals("sphrique")){
	    if(nPoint<4){
		throw new SimuException("Le nombre de points doit tre suprieur ou gal  4 pour le maillage sphrique.");
	    }

	    if(nPoint>1000){
		throw new SimuException("Le nombre de points est trop lev pour le maillage sphrique. Il doit tre infrieur  1000.");
	    }
	}


	if(gType.equals("torique")){
	    if(nPoint<3){
		throw new SimuException("Le nombre de points doit tre suprieur ou gal  3 pour le maillage torique.");
	    }

	    if(nPoint>50){
		throw new SimuException("Le nombre de points est trop lev pour le maillage torique. Il doit tre infrieur  50.");
	    }
	}



	projec = in.lireString("projec");


	String tempS = in.lireString("mode");

	if (tempS.equals("Delaunay")){
	    mode=1;
	}
	else if (tempS.equals("Voronoi")){
	    mode=2;
	}
	else if (tempS.equals("Delaunay et Voronoi")){
	    mode=3;
	}
	

	if(gType.equals("sphrique")){
	    grid = new SphereGrid(nPoint);
	    
	    
	    
	    cellList = grid.getCells();
	




	    int sup=cellList.size();


	    if( mode==1 || mode==3){

		for (int i= 0 ; i < sup; i++){

		    Cell c = (Cell)cellList.elementAt(i);

		    Vector  neigh = c.getNeighbours();
	    

		    double phiC = ((Double)(c.getSphereCoordinates()).elementAt(2)).doubleValue();

		    Vector xy;
		    
		    xy = SphereGrid.MollProjec(c);
		    
		    double xp = ((Double)xy.elementAt(0)).doubleValue();
		    double yp = ((Double)xy.elementAt(1)).doubleValue();
		    int x1 = (int)(xp*dilat)+ offsetX;
		    int y1 = (int)(yp*dilat)+ offsetY;


		    int bn = neigh.size();
		    for (int j =0; j< bn; j++){ 

			Cell n = (Cell)neigh.elementAt(j);

			double phiN = ((Double)(n.getSphereCoordinates()).elementAt(2)).doubleValue();


			

			if (projec.equals("molweide")){

			    if ( (Math.PI >=  phiC) && (phiC >= (Math.PI)/2.0) && (phiN >= Math.PI) && (phiN<=(3.0*Math.PI/2.0)))
				{
				}
			    else if ( (Math.PI >=  phiN) && (phiN >= (Math.PI)/2.0) && (phiC >= Math.PI) && (phiC<=(3.0*Math.PI/2.0)) )
				{
				}
			    else{
				Vector xyN;
			
				xyN = SphereGrid.MollProjec(n);
			
			
				double xpN = ((Double)xyN.elementAt(0)).doubleValue();
				double ypN = ((Double)xyN.elementAt(1)).doubleValue();
			
				int x2 = (int)(xpN*dilat) + offsetX;
				int y2 = (int)(ypN*dilat) + offsetY;
			
				int factcol = (int)(255.0/bn);
			
			
				g.drawLine(x1,y1,x2,y2);
			    }
			
			}


			if(projec.equals("vue cachee")){

			    if(phiC>=Math.PI || phiN>=Math.PI){
			    } 
			    else{
				Vector xyz =  c.getCartCoordinates();
				double cx = ((Double)xyz.elementAt(0)).doubleValue();
				double cy = ((Double)xyz.elementAt(1)).doubleValue();
				double cz = ((Double)xyz.elementAt(2)).doubleValue();

				x1 = (int)(-cx*dilat*1.5) + offsetX;
				y1 = (int)(cz*dilat*1.5) + offsetY;

				Vector xyz2 =  n.getCartCoordinates();
				double cx2 = ((Double)xyz2.elementAt(0)).doubleValue();
				double cy2 = ((Double)xyz2.elementAt(1)).doubleValue();
				double cz2 = ((Double)xyz2.elementAt(2)).doubleValue();

				
				int x2 = (int)(-cx2*dilat*1.5) + offsetX;
				int y2 = (int)(cz2*dilat*1.5) + offsetY;

				g.drawLine(x1,y1,x2,y2);

			    }

			}


		    }



		}
	    }
	    

	    if( mode==2 || mode==3){

	
		for (int i= 0 ; i< sup; i++){

		    g.setColor(Color.green);

		    Cell c = (Cell)cellList.elementAt(i);

		    Vector  vor = c.getVoronoi();

		    double phiC = ((Double)(c.getSphereCoordinates()).elementAt(2)).doubleValue();
		    

		    int bv = vor.size();
		    for (int j = 0; j< bv; j++){ 

			Point p = (Point)vor.elementAt(j);
			Point q;
		    
			if(j < (bv-1)){
			    q = (Point)vor.elementAt(j+1);
			}
			else{
			    q = (Point)vor.elementAt(0);
			}

			double phip = ((Double)(p.getSphereCoordinates()).elementAt(2)).doubleValue();
			double phiq = ((Double)(q.getSphereCoordinates()).elementAt(2)).doubleValue();

			

			if (projec.equals("molweide")){
			    if ( (Math.PI >=  phiC) && (phiC >= (Math.PI)/2.0) && (phip >= Math.PI) && (phip<=(3.0*Math.PI/2.0)))
				{
				}
			    else if ( (Math.PI >=  phip) && (phip >= (Math.PI)/2.0) && (phiC >= Math.PI) && (phiC<=(3.0*Math.PI/2.0)) )
				{
				}
			    if ( (Math.PI >=  phiq) && (phiq >= (Math.PI)/2.0) && (phip >= Math.PI) && (phip<=(3.0*Math.PI/2.0)))
				{
				}
			    else if ( (Math.PI >=  phip) && (phip >= (Math.PI)/2.0) && (phiq >= Math.PI) && (phiq<=(3.0*Math.PI/2.0)) )
				{
				}
			    else{

				Vector xyp;
			
				xyp = SphereGrid.MollProjec(p);
			
			
				double xp = ((Double)xyp.elementAt(0)).doubleValue();
				double yp = ((Double)xyp.elementAt(1)).doubleValue();
			
				int x1 = (int)(xp*dilat) + offsetX;
				int y1 = (int)(yp*dilat) + offsetY;


				Vector xyq;
			
				xyq = SphereGrid.MollProjec(q);
			
			
				double xq = ((Double)xyq.elementAt(0)).doubleValue();
				double yq = ((Double)xyq.elementAt(1)).doubleValue();
			
				int x2 = (int)(xq*dilat) + offsetX;
				int y2 = (int)(yq*dilat) + offsetY;

				g.drawLine(x1,y1,x2,y2);
			    }
			}
			if(projec.equals("vue cachee")){
			

			    if(phiC>=Math.PI || phip>=Math.PI || phiq>=Math.PI){
			    } 
			    else{
				Vector xyz =  p.getCartCoordinates();
				double cx = ((Double)xyz.elementAt(0)).doubleValue();
				double cy = ((Double)xyz.elementAt(1)).doubleValue();
				double cz = ((Double)xyz.elementAt(2)).doubleValue();

				int x1 = (int)(-cx*dilat*1.5) + offsetX;
				int y1 = (int)(cz*dilat*1.5) + offsetY;

				Vector xyz2 =  q.getCartCoordinates();
				double cx2 = ((Double)xyz2.elementAt(0)).doubleValue();
				double cy2 = ((Double)xyz2.elementAt(1)).doubleValue();
				double cz2 = ((Double)xyz2.elementAt(2)).doubleValue();

				int x2 = (int)(-cx2*dilat*1.5) + offsetX;
				int y2 = (int)(cz2*dilat*1.5) + offsetY;

				g.drawLine(x1,y1,x2,y2);

			    }
			

			}
		
		    }
		}
	    }
	}

	// sinon on trace le maillage carr
	else{
		
	    //nPoint = (int)(Math.sqrt(nPoint));
	    grid = new SquareGrid(nPoint);
	    
	    cellList = grid.getCells();


	    if( mode==1 || mode==3){
		for (int i= 0 ; i<cellList.size(); i++){

		    Cell c = (Cell)cellList.elementAt(i);
	    
		    Vector  neigh = c.getNeighbours();
	    

		    Vector xy = c.getCartCoordinates();
	
		    double xp = ((Double)xy.elementAt(0)).doubleValue();
		    double yp = ((Double)xy.elementAt(1)).doubleValue();
		    int x1 = (int)(xp*facteur*3/nPoint) + (int)(0.05*facteur);
		    int y1 = (int)(yp*facteur*3/nPoint)+ (int)(0.05*facteur);
	    
		    for (int j =0; j<neigh.size(); j++){ 

			Cell n = (Cell)neigh.elementAt(j);

		    
			Vector xyN = n.getCartCoordinates();
		    
			double xpN = ((Double)xyN.elementAt(0)).doubleValue();
			double ypN = ((Double)xyN.elementAt(1)).doubleValue();
		    
			int x2 = (int)(xpN*facteur*3/nPoint) + (int)(0.05*facteur);
			int y2 = (int)(ypN*facteur*3/nPoint) + (int)(0.05*facteur);
		    
			g.drawLine(x1,y1,x2,y2);
		    
		    }
		}
	    }

	    if( mode == 2 || mode==3){
		for (int i= 0 ; i<cellList.size(); i++){

		    g.setColor(Color.red);
		

		    Cell c = (Cell)cellList.elementAt(i);
		
		    Vector  vor = c.getVoronoi();
		
		    Vector xyc = c.getCartCoordinates();
		    double xc = ((Double)xyc.elementAt(0)).doubleValue();
		    double yc = ((Double)xyc.elementAt(1)).doubleValue();
		
		
		    for (int j =0; j<vor.size(); j++){ 

			Point p = (Point)vor.elementAt(j);
			Point q;
		    
			if(j < (vor.size()-1)){
			    q = (Point)vor.elementAt(j+1);
			}
			else{
			    q = (Point)vor.elementAt(0);
			}

		    

			Vector xyp = p.getCartCoordinates();
			
			double xp = ((Double)xyp.elementAt(0)).doubleValue();
			double yp = ((Double)xyp.elementAt(1)).doubleValue();
		
			// test periodicite pour P. Si oui, il faut "decaler le point de Voronoi"
			
			if ((xp-xc) > 0.5) {
			    xp = xp - nPoint/2.; 
			}
			else if ((xc-xp) > 0.5) {
			    xp = xp + nPoint/2.;
			}

			if ((yp-yc) > 0.5) {
			    yp = yp - nPoint/2.; 
			}
			else if ((yc-yp) > 0.5) {
			    yp = yp + nPoint/2.;
			}


			// Fin test periodicite pour P
	
			int x1 = (int)((xp*facteur*3.0)/nPoint) + (int)(0.05*facteur);
			int y1 = (int)((yp*facteur*3.0)/nPoint) + (int)(0.05*facteur);



			Vector xyq = q.getCartCoordinates();
			
			double xq = ((Double)xyq.elementAt(0)).doubleValue();
			double yq = ((Double)xyq.elementAt(1)).doubleValue();
		
			// test periodicite pour Q
			
			if ((xq-xc) > 0.5) {
			    xq = xq - nPoint/2.; 
			}
			else if ((xc-xq) > 0.5) {
			    xq = xq + nPoint/2.;
			}

			if ((yq-yc) > 0.5) {
			    yq = yq - nPoint/2.; 
			}
			else if ((yc-yq) > 0.5) {
			    yq = yq + nPoint/2.;
			}

			// Fin test periodicite pour Q
	
			
			int x2 = (int)((xq*facteur*3)/nPoint) + (int)(0.05*facteur);
			int y2 = (int)((yq*facteur*3)/nPoint) + (int)(0.05*facteur);			

 			g.drawLine(x1,y1,x2,y2);
		    
		    }
		}
	    }



	}

	out.ajouter("diagramme", img);
	
	return out;

    }

}
