
import java.util.Vector;
import java.util.Vector;

import java.lang.Math.*;

/**
 * Class which implements the Grid Interface.
 * Class enabling the case of a spherical grid
 */
public class SphereGrid implements Grid {

    private Cell root;

    private int dimx;
    private int dimy;

    private Vector listCells;

    /**
     * Constructor of SphereGrid
     * @param numbPoint the number of points used to build the spherical grid
     */
    public SphereGrid(int numbPoint){

	Point proot = new Point(0,0,0,0);
	root = new Cell(proot,"sphere");
	double probroot = 1.0/((double)(numbPoint));

	listCells = new Vector();



	// Generation of the initial tetrahedron
	Point pt1 = new Point(0        ,0        ,1        ,0);
	pt1.setName("tetra1");
	Point pt2 = new Point(0.94281  ,0        ,-0.33333 ,0);
	pt2.setName("tetra2");
	Point pt3 = new Point(-0.47141 ,0.81650  ,-0.33333 ,0);
	pt3.setName("tetra3");
	Point pt4 = new Point(-0.47141 ,-0.81650 ,-0.33333 ,0);
	pt4.setName("tetra4");

	Triangle trt1 = new Triangle(pt1, pt2, pt3);
	Triangle trt2 = new Triangle(pt1, pt3, pt4);
	Triangle trt3 = new Triangle(pt1, pt2, pt4);
	Triangle trt4 = new Triangle(pt2, pt3, pt4);

	Vector list_trt = new Vector();
	list_trt.addElement(trt1);
	list_trt.addElement(trt2);
	list_trt.addElement(trt3);
	list_trt.addElement(trt4);

	// Generation of the initial location structure
	LocationStructure graph = new LocationStructure(list_trt);

	// Construction of the location structure
	graph.build((numbPoint-4));

	listCells = Cell.cellGeneration(this, graph);

    }


        

    /** 
     * Returns the coordinates of the Mollweide projection  
     * @param c the Cell of which the Mollweide projection coordinates (x,y) are wanted
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public static Vector MollProjec(Cell c){

	Point p = c.getPoint();
	return SphereGrid.MollProjec(p);

    }


    /** 
     * Returns the coordinates of the Mollweide projection
     * @param c the Cell of which the Mollweide projection coordinates (x,y) are wanted
     * @param phi the longitude used to center the projection
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public static Vector MollProjec(Cell c, double phi){

	Point p = c.getPoint();
	return SphereGrid.MollProjec(p, phi);

    }


    /** 
     * Returns the coordinates of the Mollweide projection
     * @param c the Cell of which the Mollweide projection coordinates (x,y) are wanted
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public static Vector MollProjec(Cell c, double th, double phi){

	Point p = c.getPoint();
	return SphereGrid.MollProjec(p, th, phi);

    }



    /** 
     * Returns the coordinates of the Mollweide projection
     * @param c the Cell of which the Mollweide projection coordinates (x,y) are wanted
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public Vector projec(Cell c){
	return SphereGrid.MollProjec(c);
    }



    /** 
     * Returns the coordinates of the Mollweide projection  
     * @param p the point of which the Mollweide projection coordinates (x,y) are wanted
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public static Vector MollProjec(Point p){


	Vector coor = p.getSphereCoordinates();
	
	Double rayonO = (Double)coor.elementAt(0);
	Double thetaO = (Double)coor.elementAt(1);
	Double phiO = (Double)coor.elementAt(2);

	double rayon = rayonO.doubleValue();
	double theta = thetaO.doubleValue();
	double phi = phiO.doubleValue();

	if (phi > Math.PI){
	    phi = phi - 2 * Math.PI; 
	}

	double lambdaP = 2*Math.asin(2*((Math.PI/2.0) - theta)/Math.PI);
	double delta = 1;

	if (theta <0.00001){
	    lambdaP = Math.PI;
	}
	else if (theta>Math.PI-0.00001){
	    lambdaP = -(Math.PI);
	}
	else{
	    while (delta>0.00001){
		
		delta = -(lambdaP + Math.sin(lambdaP) - Math.PI * Math.cos(theta))
		    / (1+Math.cos(lambdaP));
		lambdaP = lambdaP + delta;
	    }
	}

	double lambda = lambdaP/2.0;
	
       
	double x = 2.0 * Math.sqrt(2) / Math.PI * phi * Math.cos(lambda);
	double y = Math.sqrt(2)* Math.sin(lambda);

	Vector result = new Vector();
	result.addElement(new Double(x));
	result.addElement(new Double(y));

	return result;

    }


    /** 
     * Returns the coordinates of the Mollweide projection centered on the longitude "ph"  
     * @param p the point of which the Mollweide projection coordinates (x,y) are wanted
     * @param ph the longitude used to center the projection
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    public static Vector MollProjec(Point p, double ph){


	Vector coor = p.getSphereCoordinates();
	
	Double rayonO = (Double)coor.elementAt(0);
	Double thetaO = (Double)coor.elementAt(1);
	Double phiO = (Double)coor.elementAt(2);

	double rayon = rayonO.doubleValue();
	double theta = thetaO.doubleValue();
	double phi = phiO.doubleValue() - ph;

	
	phi = (((phi % (2*Math.PI)) + 2*Math.PI) % (2*Math.PI));

	if (phi > Math.PI){
	    phi = phi - 2 * Math.PI; 
	}

	double lambdaP = 2*Math.asin(2*((Math.PI/2.0) - theta)/Math.PI);
	double delta = 1;

	if (theta <0.00001){
	    lambdaP = Math.PI;
	}
	else if (theta>Math.PI-0.00001){
	    lambdaP = -(Math.PI);
	}
	else{
	    while (delta>0.00001){
		
		delta = -(lambdaP + Math.sin(lambdaP) - Math.PI * Math.cos(theta))
		    / (1+Math.cos(lambdaP));
		lambdaP = lambdaP + delta;
	    }
	}

	double lambda = lambdaP/2.0;
	
       
	double x = 2.0 * Math.sqrt(2) / Math.PI * phi * Math.cos(lambda);
	double y = Math.sqrt(2)* Math.sin(lambda);

	Vector result = new Vector();
	result.addElement(new Double(x));
	result.addElement(new Double(y));

	return result;

    }



    /** 
     * Returns the coordinates of the Mollweide projection  
     * @param p the point of which the Mollweide projection coordinates (x,y) are wanted
     * @return result a Vector containing two Double objects representing the projection coordinates (x,y)
     */
    
    public static Vector MollProjec(Point p,double th, double ph){


	Vector coor = p.getSphereCoordinates();
	Vector coorC = p.getCartCoordinates();
	
	Double rayonO = (Double)coor.elementAt(0);
	Double thetaO = (Double)coor.elementAt(1);
	Double phiO = (Double)coor.elementAt(2);

	double rayon = rayonO.doubleValue();
	double theta = thetaO.doubleValue();
	double phi = phiO.doubleValue() - ph;

	double ca = (Math.PI/2.0)-th;

	double x = ((Double)coorC.elementAt(0)).doubleValue();
	double y = ((Double)coorC.elementAt(1)).doubleValue();
	double z = ((Double)coorC.elementAt(2)).doubleValue();

	double h = Math.sqrt(x*x+y*y);

	double xP = x * Math.cos(ph) + y * Math.sin(ph);
	double yP = - x * Math.sin(ph) + y * Math.cos(ph);

	double temp = xP;
	xP = xP*Math.cos(ca) + z * Math.sin(ca);
	double zP = z*Math.cos(ca) - temp*Math.sin(ca);

	rayon = Math.sqrt(xP*xP + yP*yP + zP*zP);

	theta = Math.acos(zP/rayon);

	if ((theta < 1E-10)||(Math.abs((theta - Math.PI))< 1E-10)){
	    phi = 0;
	}
	else {
	    phi = Math.atan(yP/xP);
	}
	if ((xP<0) && (h>1E-10)){
	    phi = phi + Math.PI;
	}
	phi = ((phi % (2*Math.PI) + 2*Math.PI)) % (2*Math.PI);

	if (phi > Math.PI){
	    phi = phi - 2 * Math.PI; 
	}

	double lambdaP = 2*Math.asin(2*((Math.PI/2.0) - theta)/Math.PI);
	double delta = 1;

	if (theta <0.00001){
	    lambdaP = Math.PI;
	}
	else if (theta>Math.PI-0.00001){
	    lambdaP = -(Math.PI);
	}
	else{
	    while (delta>0.00001){
		
		delta = -(lambdaP + Math.sin(lambdaP) - Math.PI * Math.cos(theta))
		    / (1+Math.cos(lambdaP));
		lambdaP = lambdaP + delta;
	    }
	}

	double lambda = lambdaP/2.0;
	
       
	double xProj = 2.0 * Math.sqrt(2) / Math.PI * phi * Math.cos(lambda);
	double yProj = Math.sqrt(2)* Math.sin(lambda);

	Vector result = new Vector();
	result.addElement(new Double(xProj));
	result.addElement(new Double(yProj));

	return result;

    }




    /**
     * Getter for the attribute "listCells"
     */
    public Vector getCells(){
	return listCells;
    }

    
    /**
     * Getter for the attribute "root"
     */
    public Cell getRoot(){
	return root;
    }


}
